/*  This file is part of the KDE project
    Copyright (C) 2004 Matthias Kretz <kretz@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

*/

#include "simpleplayer.h"
#include <kurl.h>
#include <kglobal.h>
#include <kinstance.h>
#include <kaboutdata.h>

namespace KDE
{
namespace Multimedia
{

class SimplePlayer::Private
{
	public:
		Channel * channel;
		Player * player;

		KURL url;
		Player::State state;
		float channelvolume;
		long time;
		QString title;
		QString type;
};

SimplePlayer::SimplePlayer( QObject * parent, const char * name )
	: QObject( parent, name )
	, d( new Private )
{
	connect( Factory::self(), SIGNAL( deleteYourObjects() ), SLOT( deleteYourObjects() ) );
	connect( Factory::self(), SIGNAL( recreateObjects() ), SLOT( recreateObjects() ) );
	d->channel = Factory::self()->createChannel( KGlobal::instance()->aboutData()->programName() );
	d->player = Factory::self()->createPlayer();
	d->player->setOutputChannel( d->channel );
	connect( d->player, SIGNAL( stateChanged( KDE::Multimedia::Player::State, KDE::Multimedia::Player::State ) ),
			SLOT( stateChanged( KDE::Multimedia::Player::State, KDE::Multimedia::Player::State ) ) );
	connect( d->player, SIGNAL( finished() ), SIGNAL( finished() ) );
}

SimplePlayer::~SimplePlayer()
{
	delete d->player;
	delete d->channel;
}

void SimplePlayer::play( const KURL & url )
{
	if( isPaused() && url == d->url )
	{
		d->player->play();
		return;
	}
	if( ! d->player->load( url ) )
		return;
	d->url = url;
	if( d->player->state() == Player::Stopped )
		d->player->play();
}

void SimplePlayer::pause()
{
	d->player->pause();
}

void SimplePlayer::stop()
{
	d->player->stop();
}

long SimplePlayer::totalTime() const
{
	return d->player->totalTime();
}

long SimplePlayer::currentTime() const
{
	return d->player->currentTime();
}

void SimplePlayer::seek( long ms )
{
	d->player->seek( ms );
}

float SimplePlayer::volume() const
{
	return d->channel->volume();
}

void SimplePlayer::setVolume( float v )
{
	d->channel->setVolume( v );
}

bool SimplePlayer::isPlaying() const
{
	return ( d->player->state() == Player::Playing );
}

bool SimplePlayer::isPaused() const
{
	return ( d->player->state() == Player::Paused );
}

void SimplePlayer::stateChanged( Player::State ns, Player::State os )
{
	if( os == Player::Loading && ns == Player::Stopped )
		d->player->play();
}

void SimplePlayer::deleteYourObjects()
{
	d->state = d->player->state();
	d->channelvolume = d->channel->volume();
	d->time = d->player->currentTime();
	d->title = d->channel->channelName();
	d->type = d->channel->channelType();

	d->player->stop();

	delete d->player;
	delete d->channel;
	d->player = 0;
	d->channel = 0;
}

void SimplePlayer::recreateObjects()
{
	d->channel = Factory::self()->createChannel( d->title, d->type );
	d->channel->setVolume( d->channelvolume );

	d->player = Factory::self()->createPlayer();
	d->player->setOutputChannel( d->channel );

	if( d->state != Player::NoMedia )
		if( ! d->player->load( d->url ) )
			return;
	if( d->state == Player::Playing || d->state == Player::Paused || d->state == Player::Buffering )
	{
		d->player->play();
		d->player->seek( d->time );
		if( d->state == Player::Paused )
			d->player->pause();
	}
}

}} // namespaces

#include "simpleplayer.moc"

// vim: sw=4 ts=4 noet
